# encoding: utf-8
"""
@project: djangoModel->__init__
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 抽象库、监听者模式、策略模式设计。
@note: 该目录下面所有文件不专注页具体业务，不专注服务于主框架。实现面向零散功能的逻辑代码，
note 如：微信登录和框架镖行业务的登录关系,镖行登录调用微信登录。 所有类不可模型。
@created_time: 2023/6/14 14:05
"""
from .j_valuation.j_valuation import JValuation

__all__ = [
    JValuation
]



# encoding: utf-8
"""
@project: djangoModel->j_expression
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 公式类
@created_time: 2023/6/14 14:41
"""
import math

from xj_enroll.utils.custom_tool import write_to_log
from .j_base_expression import JBaseExpression


class CalculateExpression(JBaseExpression):
    name = ""
    patt = "\([^\(]*?\)"

    def process(self, expression_string):
        try:
            return self.safe_eval(expression_string)  # 加加减乘除
        except ZeroDivisionError:
            return 0  # 0/n  的情况
        except Exception as e:
            write_to_log(level="error", prefix="计价异常（SUMExpression）", err_obj=e)
            return expression_string


class IFExpression(JBaseExpression):
    """
    布尔值判断、三元运算
    """
    name = "IF"
    patt = "IF\([^\(]*?\)"

    def process(self, expression, *args, ):
        bool_expression, yes_expression, no_expression = expression.replace(IFExpression.name, "").replace("(", "").replace(")", "").split(",")
        if CalculateExpression().process(bool_expression):
            return yes_expression
        else:
            return no_expression


class SUMExpression(JBaseExpression):
    """
    求和计算、（多参数在服务中获取）
    """
    name = "SUM"
    patt = "SUM\([^\(]*?\)"  # SUM(a,b,c) == a+b+c

    def process(self, expression):
        try:
            args = expression.replace(self.name, "").replace("(", "").replace(")", "").split(",")
            result = 0
            for i in args:
                result = self.safe_eval(str(result) + "+" + str(i))
            return result
        except Exception as e:
            write_to_log(level="error", prefix="计价异常（SUMExpression）", err_obj=e)
            return 0


class CEILExpression(JBaseExpression):
    """
    取整公式
    """
    name = "CEIL"
    patt = "CEIL\([^\(]*?\)"  # ROUND(num, index, step)

    def process(self, expression):
        try:
            expression = expression.replace(self.name, "").replace("(", "").replace(")", "")
            return math.ceil(self.safe_eval(expression))
        except Exception as e:
            write_to_log(level="error", prefix="计价异常（SUMExpression）", err_obj=e)
            return 0



# encoding: utf-8
"""
@project: djangoModel->j_valuation
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 计价服务类
@created_time: 2022/10/13 16:58
"""

import re

from xj_enroll.utils.custom_tool import write_to_log
from .j_base_expression import JBaseExpression
from .j_expressions import CalculateExpression


class JValuation(JBaseExpression):
    """
    计算入口类，计算器 对外暴露的类
    """

    def process(self, expression_string):
        # 公式合法性校验：expression_string
        if expression_string is None:
            return "", None
        if not len(re.findall("\(", expression_string)) == len(re.findall("\)", expression_string)):
            write_to_log(prefix="计价公式配置错误，请及时检查", content="语法错误，括号应该成对存在")
            return None, "语法错误，括号应该成对存在"
        # 公式字符串,符号预处里预处理
        expression_string = expression_string.upper().replace(" ", "").replace("=", "==").replace(">==", ">=").replace("<==", "<=")

        # 把自定义公式计算出结果进行替换，得出基础的加减乘除公式字符拆
        parsed_expression_string = self.parse_expression(expression_string)

        # 计算基础加减乘除，然后得出结果并返回
        result = CalculateExpression().process(parsed_expression_string)
        return result, None
